@@ -16,6 +16,7 @@
 #include "QLabelW.h"
 #include "QLabelImage.h"
 
+
 #ifdef Q_WS_MAEMO_5
 #include <QDBusInterface>
 #include <QDBusConnection>
@@ -482,13 +483,29 @@ void DetailWindow::_addField (TargetField * field) {
 	}
 	else
 	if (field->type == TYPE_PASSWORD) {
+#ifdef OPEN_SOURCE_VERSION
 		QLineEdit * nameLineEdit = new QLineEdit(this);
 		nameLineEdit->setText (field->value);
 		nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
 		this->values.append (nameLineEdit);
 		componentLayout->addWidget(nameLineEdit, pos, 1, 1, 1);
 		nameLineEdit->setFocusPolicy(Qt::StrongFocus);
-		//nameLineEdit->setContentsMargins(0, 0, 20, 0);
+#else
+		QLineEdit * nameLineEdit = new QLineEdit(this);
+		nameLineEdit->setText (field->value);
+		nameLineEdit->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
+		this->values.append (nameLineEdit);
+		QHBoxLayout * ll = new QHBoxLayout;
+		ll->addWidget(nameLineEdit);
+		QPushButton * generate = new QPushButton(tr("Generate"), this);
+		ll->addWidget(generate);
+		generate->setUserData(0, new ButtonInfo2 (nameLineEdit)); //ObjectName(field->value);
+		signalMapper->setMapping(generate, generate);
+		QObject::connect(generate, SIGNAL(clicked()), signalMapper, SLOT(map()));
+
+		componentLayout->addLayout(ll, pos, 1, 1, 1);
+		nameLineEdit->setFocusPolicy(Qt::StrongFocus);
+#endif
 	}
 	else
 	if (field->type == TYPE_INTEGER) {
@@ -581,11 +598,11 @@ void DetailWindow::showFields () {
 		this->labels.clear();
 	}
 
-	QSignalMapper * signalMapper = NULL;
-	if (!edit) {
-		signalMapper = new QSignalMapper(this);
+	signalMapper = new QSignalMapper(this);
+	if (edit)
+		QObject::connect(signalMapper, SIGNAL(mapped(QWidget *)), this, SLOT(passGen(QWidget *)));
+	else
 		QObject::connect(signalMapper, SIGNAL(mapped(QWidget *)), this, SLOT(clickPass(QWidget *)));
-	}
 
 	componentLayout->setColumnStretch(0, 10);
 	componentLayout->setColumnStretch(1, 25);
@@ -691,6 +708,22 @@ void DetailWindow::showFields () {
 	}
 }
 
+void DetailWindow::passGen (QWidget * _wid) {
+	QPushButton * b = (QPushButton *)_wid;
+
+	if (createPasswordDlg.isNull()) {
+		createPasswordDlg = new CreatePasswordDialog(this);
+		createPasswordDlg->setMinimumSize(size().width(),size().height()-100);
+		if (createPasswordDlg->exec() == QDialog::Accepted) {
+			((ButtonInfo2 *)b->userData(0))->nameLineEdit->setText(createPasswordDlg->getResult());
+			((ButtonInfo2 *)b->userData(0))->nameLineEdit->setModified(true);
+		}
+
+		delete createPasswordDlg;
+		createPasswordDlg = 0;
+	}
+}
+
 void DetailWindow::clickPass (QWidget * _wid) {
 	QPushButton * b = (QPushButton *)_wid;
 
@@ -1402,8 +1435,14 @@ void DetailWindow::showN () {
 	if (!createFieldDlg.isNull()) {
 		createFieldDlg->show();
 	}
+	if (!createPasswordDlg.isNull()) {
+		createPasswordDlg->show();
+	}
 }
 void DetailWindow::hideN () {
+	if (!createPasswordDlg.isNull()) {
+		createPasswordDlg->hide();
+	}
 	if (!createFieldDlg.isNull()) {
 		createFieldDlg->hide();
 	}
