@@ -20,6 +20,7 @@
  * TODO be able to copy sms and others messages (and contacts?) into wallet.
  * TODO add configuration file to backup
  * TODO get card properties from scanner
+ * TODO show target summary translate
  */
 
 #include "utils.h"
@@ -81,35 +82,38 @@ class eWallet::MyDelegate : public QStyledItemDelegate  {
 		else {
 			QVariant val = index.data(Qt::DecorationRole);
 			Target * target = parentWallet.list.at(_index.toInt());
+
 			QString description = target->getSummary();
+			// TODO translate ?? tr(types_string[this->parent->types->at(i)->id - 1])
+
 
 			QIcon pixmap;
-			if (val.isNull()) {
-				qDebug () << "getting image " << title;
-
-				if (target->getPictureSmall() != NULL) {
-					pixmap = QPixmap::fromImage(*target->getPictureSmall());
-
-					// at this moment we don't need anymore original images!
-					target->freeResources();
-				} else {
-					QString tmp = ":/resources/empty_";
-					tmp += target->type.simplified().replace(' ', '_');
-					tmp += ".png";
-
-					if (QResource(tmp).isValid()) {
-						pixmap = QPixmap(tmp);
-					} else {
-						pixmap = QPixmap(":/resources/empty_Others.png");
-					}
-				}
-
-				parentWallet.fnListWidget->item(_index.toInt())->setIcon(pixmap);
-				//index.data(Qt::DecorationRole) = pixmap;
-			}
-			else {
+//			if (val.isNull()) {
+//				qDebug () << "getting image " << title;
+//
+//				if (target->getPictureSmall() != NULL) {
+//					pixmap = QPixmap::fromImage(*target->getPictureSmall());
+//
+//					// at this moment we don't need anymore original images!
+//					target->freeResources();
+//				} else {
+//					QString tmp = ":/resources/empty_";
+//					tmp += target->type.simplified().replace(' ', '_');
+//					tmp += ".png";
+//
+//					if (QResource(tmp).isValid()) {
+//						pixmap = QPixmap(tmp);
+//					} else {
+//						pixmap = QPixmap(":/resources/empty_Others.png");
+//					}
+//				}
+//
+//				parentWallet.fnListWidget->item(_index.toInt())->setIcon(pixmap);
+//				//index.data(Qt::DecorationRole) = pixmap;
+//			}
+//			else {
 				pixmap = qvariant_cast<QIcon>(val);
-			}
+//			}
 
 			QRect r = option.rect.adjusted(0, 0, option.decorationSize.width(), option.decorationSize.height());
 			pixmap.paint(painter, 5, r.top () + 5, SMALL_WIDTH, SMALL_HEIGH, option.decorationAlignment, QIcon::Normal);
@@ -211,6 +215,9 @@ eWallet::eWallet(QWidget *parent) :
 	QMainWindow(parent) {
 	setWindowTitle("Wallet");
 
+//	this->setAttribute(Qt::WA_DeleteOnClose);
+//	connect(this, SIGNAL(destroyed(QObject*)), this, SLOT(walletDestroyed(QObject*)));
+
 	//    INTERNAL_WIDTH = settings.value("internalImage/WIDTH", _INTERNAL_WIDTH).toInt();
 	//    INTERNAL_HEIGH = settings.value("internalImage/HEIGH", _INTERNAL_HEIGH).toInt();
 
@@ -503,6 +510,19 @@ void eWallet::showList(bool interactive) {
 	// Custom list box
 	//vboxlayout->addWidget(backButton);
 
+	if(!threadLoad.isNull()) {
+		disconnect(threadLoad, SIGNAL(loadImageFor(int)), this, SLOT(loadImageFor(int)));
+		qDebug () << "Wait for thread create";
+
+		while (threadLoad->isRunning()) {
+			threadLoad->wait(100);
+			QCoreApplication::processEvents ();
+		}
+
+		delete threadLoad;
+		threadLoad = 0;
+	}
+
 	fnListWidget = new QListWidget(this);
 	this->originalDelegate = new MyDelegate(*this, fnListWidget);// TODO fnListWidget->itemDelegate();
 	this->newDelegate = new MyDelegate(*this, fnListWidget);
@@ -804,13 +824,7 @@ void eWallet::lockWallet() {
 void eWallet::deleteDB() {
 	if (messageBox.isNull()) {
 		messageBox = new QMessageBox(this);
-		int
-				b =
-						messageBox->information(
-								this,
-								"Wallet",
-								tr(
-										"Are you sure you want to remove all your data? You will lost all your data"),
+		int	b = messageBox->information(this,"Wallet",tr("Are you sure you want to remove all your data? You will lost all your data"),
 								QMessageBox::Yes, QMessageBox::No);
 		if (b == QMessageBox::Yes) {
 			::dbManager->reset();
@@ -881,9 +895,23 @@ bool eWallet::openDataBase(const QString & pas) {
 }
 
 eWallet::~eWallet() {
+	if(!threadLoad.isNull()) {
+		disconnect(threadLoad, SIGNAL(loadImageFor(int)), this, SLOT(loadImageFor(int)));
+		qDebug () << "Wait for thread destroy";
+
+		while (threadLoad->isRunning()) {
+			threadLoad->wait(100);
+			QCoreApplication::processEvents ();
+		}
+
+		//delete threadLoad;
+		//threadLoad = 0;
+	}
+	qDebug () << "deleting group";
 	if (currentGroup != NULL)
 		delete currentGroup;
 	//delete ::dbManager;
+	qDebug () << "end ~eWallet";
 }
 
 /*
@@ -1210,6 +1238,10 @@ bool sortStringDES(const QString f, const QString g) {
 
 QPixmap eWallet::getImage(Target * target) {
 	QPixmap image;
+	if (target == NULL) {
+		image = QPixmap(":/resources/loading.png");
+	}
+	else
 	if (target->getPictureSmall() != NULL) {
 		image = QPixmap::fromImage(*target->getPictureSmall());
 
@@ -1232,6 +1264,19 @@ QPixmap eWallet::getImage(Target * target) {
 void eWallet::showGroups(bool reset) {
 	//qDebug () << "show Groups " << reset;
 
+	if(!threadLoad.isNull()) {
+		disconnect(threadLoad, SIGNAL(loadImageFor(int)), this, SLOT(loadImageFor(int)));
+		qDebug () << "Wait for thread groups";
+
+		while (threadLoad->isRunning()) {
+			threadLoad->wait(100);
+			QCoreApplication::processEvents ();
+		}
+
+		delete threadLoad;
+		threadLoad = 0;
+	}
+
 	fnListWidget->setItemDelegate(originalDelegate);
 
 	int current = fnListWidget->currentRow();
@@ -1289,12 +1334,41 @@ void eWallet::showInfo(bool reset) {
 	}
 }
 
+
+void eWallet::loadImageFor(int index) {
+	if (index >= 0 && index < list.size()) {
+		qDebug () << "loading image for " << index;
+
+		//QCoreApplication::sendPostedEvents ();
+
+		Target * target = list.at(index);
+		QListWidgetItem * listitem = fnListWidget->item(index);
+		listitem->setIcon(getImage(target));
+
+		QCoreApplication::processEvents ();
+		//QApplication::instance()->processEvents();
+	}
+}
+
 void eWallet::showTargets(bool reset, const QString * filter) {
 #ifdef Q_WS_MAEMO_5
 	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
 #endif
 	//qDebug () << "show Targets " << reset;
 
+	if(!threadLoad.isNull()) {
+		disconnect(threadLoad, SIGNAL(loadImageFor(int)), this, SLOT(loadImageFor(int)));
+		qDebug () << "Wait for thread showTargets";
+
+		while (threadLoad->isRunning()) {
+			threadLoad->wait(100);
+			QCoreApplication::processEvents ();
+		}
+
+		delete threadLoad;
+		threadLoad = 0;
+	}
+
 	fnListWidget->setItemDelegate(newDelegate);
 
 	int current = fnListWidget->currentRow();
@@ -1334,15 +1408,12 @@ void eWallet::showTargets(bool reset, const QString * filter) {
 		Target * target = list.at(i);
 
 		//QListWidgetItem* listitem = new QListWidgetItem(getImage(target),target->getName());
-		QListWidgetItem* listitem = new QListWidgetItem(target->getName());
+		QListWidgetItem* listitem = new QListWidgetItem(getImage(NULL), target->getName());
 		listitem->setData(Qt::UserRole + 1, QVariant (i));
 
 		fnListWidget->addItem(listitem);
-
 	}
 
-	//fnListWidget->setItemDelegate(new MyDelegate(fnListWidget));
-
 	filterBy(currentFilter);
 
 	if (current >= 0 && current <= list.size())
@@ -1355,6 +1426,11 @@ void eWallet::showTargets(bool reset, const QString * filter) {
 #ifdef Q_WS_MAEMO_5
 	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
 #endif
+
+	threadLoad = new MyLoadImageThread;
+	threadLoad->size = list.size(); //parentWallet = this;
+	connect(threadLoad, SIGNAL(loadImageFor(int)), this, SLOT(loadImageFor(int)), Qt::BlockingQueuedConnection);
+	threadLoad->start();
 }
 
 void eWallet::createMenu() {
@@ -1633,6 +1709,19 @@ void eWallet::deleteTarget(Target * target) {
 }
 
 void eWallet::windowDestroyedExtra(QObject* obj) {
+	if(!threadLoad.isNull()) {
+		disconnect(threadLoad, SIGNAL(loadImageFor(int)), this, SLOT(loadImageFor(int)));
+		qDebug () << "Wait for thread windowdestroy";
+
+		while (threadLoad->isRunning()) {
+			threadLoad->wait(100);
+			QCoreApplication::processEvents ();
+		}
+
+		delete threadLoad;
+		threadLoad = 0;
+	}
+
 	disconnect(obj, SIGNAL(destroyed(QObject*)), this,
 			SLOT(windowDestroyedExtra(QObject*)));
 
@@ -1661,6 +1750,31 @@ void eWallet::windowDestroyed(QObject* obj) {
 	delete detailWindow;
 	detailWindow = 0;
 }
+
+void eWallet::walletDestroyed(QObject* obj) {
+	if(!threadLoad.isNull()) {
+		disconnect(threadLoad, SIGNAL(loadImageFor(int)), this, SLOT(loadImageFor(int)));
+
+		qDebug () << "Wait for thread wallet destroy";
+
+		while (threadLoad->isRunning()) {
+			threadLoad->wait(100);
+			QCoreApplication::processEvents ();
+		}
+
+		delete threadLoad;
+		threadLoad = 0;
+	}
+
+	disconnect(obj, SIGNAL(destroyed(QObject*)), this,
+			SLOT(walletDestroyed(QObject*)));
+
+	qDebug () << "destroy wallet ";
+
+//	delete ::dbManager;
+//	::dbManager = NULL;
+}
+
 void eWallet::windowDestroyedEdit(QObject* obj) {
 	disconnect(obj, SIGNAL(destroyed(QObject*)), this,
 			SLOT(windowDestroyedEdit(QObject*)));
