@@ -21,6 +21,24 @@
 
 extern DatabaseManager * dbManager;
 
+class MyLoadImageThread: public QThread {
+	Q_OBJECT
+
+	signals:
+		void loadImageFor (int index);
+	public:
+		int size;
+
+		void run() {
+			qDebug () << "loading images START";
+			for (int i = 0; i < size; i++) {
+				qDebug () << "loading image " << i;
+				emit loadImageFor(i);
+			}
+			qDebug () << "loading images FINISH";
+		}
+};
+
 class eWallet: public QMainWindow
 {
 	Q_OBJECT
@@ -62,6 +80,8 @@ public slots:
 private:
 	class MyDelegate;
 
+	QPointer<MyLoadImageThread> threadLoad;
+
 	QAbstractItemDelegate * originalDelegate;
 	QAbstractItemDelegate * newDelegate;
 
@@ -87,11 +107,14 @@ private:
     void reallyChangePassword (const QString & newPassword);
 
 private slots:
+	void loadImageFor (int index);
+
 	void finalPassword (const QString pas);
 	void finalChangePassword (const QString pas);
 	void finalUnlock (const QString pas);
 
 	void orientationChanged(int);
+    void walletDestroyed(QObject* obj = 0);
     void windowDestroyed(QObject* obj = 0);
     void windowDestroyedExtra(QObject* obj = 0);
     void windowDestroyedEdit(QObject* obj = 0);
